<?php
require_once('vendor/flexi/flexi.php');

class StudiFormularPlugin extends AbstractStudIPSystemPlugin
{
	var $template_factory;
	var $db;

	function StudiFormularPlugin()
	{
		parent::AbstractStudIPSystemPlugin();
		$this->setPluginiconname('images/studiformular.gif');

		# Navigationsreiter erzeugen
		$nav = new PluginNavigation();
		$nav->setDisplayname($this->getPluginname());
		$this->setNavigation($nav);
	}

	function setPluginpath($path)
	{
		parent::setPluginpath($path);
		$this->buildMenu();
	}

	function buildMenu() {
		$user = $this->getUser();
		$permissions = $user->getPermission();

		$tab = null;
		if ($permissions->hasRootPermission() or $permissions->isStudent() or $permissions->hasTeacherPermission())
		{
			$tab = new PluginNavigation();
			$tab->setDisplayname(_('Formulare'));
		}
		$this->setNavigation($tab);
	}

	/**
	 * Liefert den Namen des Plugins zurck.
	 * @return string Der Name des Plugins
	 */
	function getPluginname()
	{
		return _('Studenten-Formular Plugin');
	}

	function show()
	{
		$this->template_factory = new Flexi_TemplateFactory($this->getPluginpath().'/templates');
		$this->db = new DB_Seminar;

		$user = $this->getUser();
		$permission = $user->getPermission();

		if ($permission->hasRootPermission())
		{
			$this->show_root();
		}
		elseif ($permission->hasTeacherPermission())
		{
			$this->show_teacher();
		}
		else
		{
			$this->show_student();
		}
	}

	function show_root()
	{
		if (!empty($_POST))
		{
			if (empty($_POST['name']) or empty($_POST['url']) or ($_POST['categoryid']=='-1' and empty($_POST['new_category'])))
			{
				$this->displayError('Bitte fllen Sie alle bentigten Felder korrekt aus');
			}
			else
			{
				$categoryid = ($_POST['categoryid']=='-1')
					? $this->createCategory($_POST['new_category'])
					: $_POST['categoryid'];
				if ($this->insertFormular($_POST['url'], $_POST['name'], $_POST['description'], $_POST['status'], $categoryid))
				{
					$this->displaySuccess('Das Formular wurde erfolgreich eingefgt');
				}
				else
				{
					$this->displayError('Das Formular konnte nicht eingetragen werden');
				}
			}
		}
		elseif (!empty($_GET['delete']))
		{
			$this->db->queryf("DELETE FROM studiform_urls WHERE urlid=%d LIMIT 1", $_GET['delete']);
			if ($this->db->affected_rows()>0)
			{
				$this->displaySuccess('Das Formular wurde erfolgreich gelscht');
			}
			else
			{
				$this->displayError('Das Formular konnte nicht gelscht werden');
			}
		}
		elseif (!empty($_GET['view']))
		{
			switch ($_GET['view'])
			{
				case 'student':
					$this->show_student();
				break;
				case 'teacher':
					$this->show_teacher();
				break;
			}
			return;
		}


		$template =& $this->template_factory->open('root');
		$template->set_attribute('links', array(
			'delete'=>PluginEngine::getLink($this).'&delete=',
			'submit'=>PluginEngine::getLink($this),
			'reset'=>PluginEngine::getLink($this),
			'student_view'=>PluginEngine::getLink($this).'&view=student',
			'teacher_view'=>PluginEngine::getLink($this).'&view=teacher'
		));
		$template->set_attribute('categories', $this->getAllCategories());
		$template->set_attribute('student_list', $this->getUrls('student'));
		$template->set_attribute('teacher_list', $this->getUrls('teacher'));
		echo $template->render();
	}

	function createCategory($name)
	{
		$this->db->queryf("INSERT INTO studiform_categories (name) VALUES ('%s')", $name);
		$this->db->queryf("SELECT MAX(categoryid) AS categoryid FROM studiform_categories");
		$this->db->next_record();
		$categoryid = $this->db->f('categoryid');
		$this->db->free();

		return $categoryid;
	}

	function insertFormular($url, $name, $description, $status, $categoryid)
	{
		$this->db->queryf("INSERT INTO studiform_urls (name, description, url, status, categoryid) VALUES ('%s', '%s', '%s', '%s', %d)",
			$name,
			$description,
			$url,
			$status,
			$categoryid
		);
		return $this->db->affected_rows()>0;
	}

	function show_teacher()
	{
		$this->showByStatus('teacher');
	}

	function show_student()
	{
		$this->showByStatus('student');
	}

	function showByStatus($status)
	{
		$template = $this->template_factory->open('links');
		$template->set_attribute('links', $this->getUrls($status));
		echo $template->render();
	}

	function getUrls($status='teacher')
	{
		$categories = array();
		$this->db = new DB_Seminar;

		$this->db->queryf("SELECT urlid, u.name AS name, description, url, c.name AS category FROM studiform_urls AS u LEFT JOIN studiform_categories AS c USING(categoryid) WHERE status='%s' ORDER BY c.name ASC, u.name ASC", $status);
		while ($this->db->next_record())
		{
			$hash = md5($this->db->f('category'));
			if (!isset($categories[$hash]))
			{
				$categories[$hash] = array(
					'name'=>$this->db->f('category'),
					'urls'=>array()
				);
			}
			array_push($categories[$hash]['urls'], array(
				'id'=>$this->db->f('urlid'),
				'name'=>$this->db->f('name'),
				'description'=>$this->db->f('description'),
				'url'=>$this->db->f('url')
			));
		}
		$this->db->free();

		return $categories;
	}

	function getAllCategories()
	{
		$categories = array();

		$this->db->queryf("SELECT categoryid, name FROM studiform_categories ORDER BY name ASC");
		while ($this->db->next_record())
		{
			array_push($categories, array(
				'id'=>$this->db->f('categoryid'),
				'name'=>$this->db->f('name')
			));
		}
		$this->db->free();

		return $categories;
	}

	function _display()
	{
		$arguments = func_get_args();
		$type = array_shift($arguments);
		$message = array_shift($arguments);

		if (!is_array($message))
		{
			$message = array(vsprintf($message, $arguments));
		}
		$message = array_map(create_function('$a', 'return rtrim($a, "!. ");'), $message);

		$template =& $this->template_factory->open('_'.$type);
		$template->set_attribute($type, $message);
		echo $template->render();
	}

	function displayError()
	{
		$arguments = func_get_args();
		array_unshift($arguments, 'error');
		call_user_func_array(array($this, '_display'), $arguments);
	}

	function displaySuccess()
	{
		$arguments = func_get_args();
		array_unshift($arguments, 'success');
		call_user_func_array(array($this, '_display'), $arguments);
	}

	# Kompatiblitt zu Pluginschnittstellennderungen
	function actionShow()
	{
			return $this->show();
	}
}
?>